@@ -27,14 +27,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 //void se_FetchAndStoreSDLInput();
 
+#include "rSDL.h"
+
 #include "tConfiguration.h"
 
 // floor mirror
+#ifndef DEDICATED
 static REAL sr_floorMirror_strength=.1;
 static tSettingItem<REAL> f_m("FLOOR_MIRROR_INT",sr_floorMirror_strength);
 
-#ifndef DEDICATED
-
 #include "tEventQueue.h"
 #include "uInputQueue.h"
 #include "eTess2.h"
@@ -49,8 +50,10 @@ static tSettingItem<REAL> f_m("FLOOR_MIRROR_INT",sr_floorMirror_strength);
 #include "eAdvWall.h"
 #include "eFloor.h"
 #include "ePath.h"
+#include "eGrid.h"
 #include "eDebugLine.h"
 #include "tDirectories.h"
+#include "eRectangle.h"
 
 #define eWall_h 4
 #define view_h 2.7
@@ -65,8 +68,8 @@ REAL lower_height=50;
 
 #ifndef DEDICATED
 
-static rTexture sky(rTEX_FLOOR,"textures/sky.png",1,1,true);
-static rTexture sky_moviepack(rTEX_FLOOR,"moviepack/sky.png",1,1,true);
+static rFileTexture sky(rTextureGroups::TEX_FLOOR,"textures/sky.png",1,1,true);
+static rFileTexture sky_moviepack(rTextureGroups::TEX_FLOOR,"moviepack/sky.png",1,1,true);
 
 extern bool sg_MoviePack();
 
@@ -88,7 +91,38 @@ static void sky_select(){
 // if the rip bug is activated, don't use the rim to draw the floor
 extern short se_bugRip;
 
-static void infinity_xy_plane(const eCoord &pos,const eCoord &dir,REAL h=0){
+// passes a vertex with z-projected texture coordinates to OpenGL
+static inline void TexVertex( REAL x, REAL y, REAL h)
+{
+    glTexCoord2f(x, y);
+    glVertex3f  (x, y, h);
+}
+
+// renders a finite rectangle
+static void finite_xy_plane( const eCoord &pos,const eCoord &dir,REAL h, eRectangle rect )
+{
+    // expand plane to camera position to avoid embarrasing reflection bug
+    if ( sr_floorMirror )
+        rect.Include( pos );
+
+    // fetch rectangle coordinates
+    REAL lx = rect.GetLow().x;
+    REAL ly = rect.GetLow().y;
+    REAL hx = rect.GetHigh().x;
+    REAL hy = rect.GetHigh().y;
+
+    // draw rectangle as triangle fan (good for avoiding artefacts near pos)
+    BeginTriangleFan();
+    TexVertex( pos.x-dir.x, pos.y-dir.y, h );
+    TexVertex(lx, ly, h);
+    TexVertex(lx, hy, h);
+    TexVertex(hx, hy, h);
+    TexVertex(hx, ly, h);
+    TexVertex(lx, ly, h);
+    RenderEnd();
+}
+
+static void infinity_xy_plane(eCoord const & pos, const eCoord &dir,REAL h=0){
     glEdgeFlag(GL_FALSE);
 
     bool use_rim=false;
@@ -100,24 +134,34 @@ static void infinity_xy_plane(const eCoord &pos,const eCoord &dir,REAL h=0){
     if ( se_bugRip )
         use_rim=false;
 
+    // always use the rim if infinity rendering is turned off
+    use_rim |= !sr_infinityPlane;
+
     if (use_rim){
-        BeginTriangles();
-        for(int i=se_rimWalls.Len()-1;i>=0;i--){
-            eCoord p1=se_rimWalls(i)->EndPoint(0);
-            eCoord p2=se_rimWalls(i)->EndPoint(1);
+        /*
+          // the rim wall based rendering does not work properly for shaped arenas, so
+          // it's been replaced.
 
-            glTexCoord2f(pos.x, pos.y);
-            glVertex3f  (pos.x, pos.y, h);
+                BeginTriangles();
+                for(int i=se_rimWalls.Len()-1;i>=0;i--){
+                    eCoord p1=se_rimWalls(i)->EndPoint(0);
+                    eCoord p2=se_rimWalls(i)->EndPoint(1);
 
-            glTexCoord2f(p1.x, p1.y);
-            glVertex3f  (p1.x, p1.y, h);
+                    glTexCoord2f(pos.x, pos.y);
+                    glVertex3f  (pos.x, pos.y, h);
 
-            glTexCoord2f(p2.x, p2.y);
-            glVertex3f  (p2.x, p2.y, h);
-        }
-        RenderEnd();
+                    glTexCoord2f(p1.x, p1.y);
+                    glVertex3f  (p1.x, p1.y, h);
+
+                    glTexCoord2f(p2.x, p2.y);
+                    glVertex3f  (p2.x, p2.y, h);
+                }
+                RenderEnd();
+        */
+        finite_xy_plane( pos, dir, h, eWallRim::GetBounds() );
     }
-    else{
+    else
+    {
         if (!sr_infinityPlane)
             zero=.001;
 
@@ -150,6 +194,7 @@ static REAL z=0;
 
 int           eGrid::NumberOfCameras(){return cameras.Len();}
 const eCoord& eGrid::CameraPos(int i){return cameras(i)->CameraPos();}
+eCoord eGrid::CameraGlancePos(int i){return cameras(i)->CameraGlancePos();}
 const eCoord& eGrid::CameraDir(int i){return cameras(i)->CameraDir();}
 REAL          eGrid::CameraHeight(int i){return cameras(i)->CameraZ();}
 
@@ -158,7 +203,7 @@ REAL          eGrid::CameraHeight(int i){return cameras(i)->CameraZ();}
 
 eWall *displayed_eWall=0;
 
-void draw_eWall(eGrid* grid, int v,int i, REAL& zNear)
+void draw_eWall(eGrid* grid, int v,int i, REAL& zNear, eCamera const * cam)
 {
     if (i<se_wallsVisible[v].Len())
     {
@@ -205,12 +250,12 @@ void draw_eWall(eGrid* grid, int v,int i, REAL& zNear)
                     }
                 }
 
-                displayed_eWall->Render();
+                displayed_eWall->Render(cam);
             }
             displayed_eWall=0;
 
-            draw_eWall(grid,v,tHeapBase::UpperL(i),zNear);
-            draw_eWall(grid,v,tHeapBase::UpperR(i),zNear);
+            draw_eWall(grid,v,tHeapBase::UpperL(i),zNear,cam);
+            draw_eWall(grid,v,tHeapBase::UpperR(i),zNear,cam);
 
 #ifdef DEBUG
         }
@@ -219,7 +264,7 @@ void draw_eWall(eGrid* grid, int v,int i, REAL& zNear)
 }
 
 
-void paint_sr_lowerSky(eGrid *grid, int viewer,bool sr_upperSky){
+void paint_sr_lowerSky(eGrid *grid, int viewer,bool sr_upperSky, eCoord const & camPos){
     TexMatrix();
     glLoadIdentity();
     glScalef(.005,.005,.005);
@@ -242,7 +287,7 @@ void paint_sr_lowerSky(eGrid *grid, int viewer,bool sr_upperSky){
     }
     if (sa>0){
         glColor4f(1,1,1,sa);
-        infinity_xy_plane(grid->CameraPos(viewer),grid->CameraDir(viewer),lower_height);
+        infinity_xy_plane(camPos,grid->CameraDir(viewer),lower_height);
     }
     if (!sr_upperSky && sr_alphaBlend)
         glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
@@ -252,7 +297,7 @@ void eGrid::display_simple( int viewer,bool floor,
                             bool sr_upperSky,bool sr_lowerSky,
                             REAL flooralpha,
                             bool eWalls,bool gameObjects,
-                            REAL& zNear ){
+                            REAL& zNear){
     /*
     static GLfloat S[]={1,0,0,0};
     static GLfloat T[]={0,1,0,0};
@@ -284,6 +329,9 @@ void eGrid::display_simple( int viewer,bool floor,
 
     glDisable(GL_CULL_FACE);
 
+    eCoord camPos = CameraGlancePos( viewer );
+    // eWallRim::Bound( camPos, 10 );
+
     if (sr_upperSky || se_BlackSky()){
         if (se_BlackSky()){
             //glDisable(GL_TEXTURE);
@@ -292,7 +340,7 @@ void eGrid::display_simple( int viewer,bool floor,
             glColor3f(0,0,0);
 
             if ( z < lower_height )
-                infinity_xy_plane(this->CameraPos(viewer),this->CameraDir(viewer),lower_height);
+                infinity_xy_plane(camPos, this->CameraDir(viewer),lower_height);
 
             glEnable(GL_TEXTURE_2D);
         }
@@ -306,17 +354,23 @@ void eGrid::display_simple( int viewer,bool floor,
             glColor3f(.5,.5,1);
 
             if ( z < upper_height )
-                infinity_xy_plane(this->CameraPos(viewer),this->CameraDir(viewer),upper_height);
+                infinity_xy_plane(camPos, this->CameraDir(viewer),upper_height);
         }
     }
 
     if (sr_lowerSky && !sr_highRim){
-        paint_sr_lowerSky(this, viewer,sr_upperSky);
+        paint_sr_lowerSky(this, viewer,sr_upperSky, camPos);
     }
 
     if (floor){
         su_FetchAndStoreSDLInput();
-        switch(sr_floorDetail){
+        int floorDetail = sr_floorDetail;
+
+        // no multitexturing without alpha blending
+        if ( !sr_alphaBlend && floorDetail > rFLOOR_TEXTURE )
+            floorDetail = rFLOOR_TEXTURE;
+
+        switch(floorDetail){
         case rFLOOR_OFF:
             break;
         case rFLOOR_GRID:
@@ -365,7 +419,7 @@ void eGrid::display_simple( int viewer,bool floor,
             se_glFloorTexture();
             se_glFloorColor(flooralpha);
 
-            infinity_xy_plane(CameraPos(viewer),CameraDir(viewer));
+            infinity_xy_plane( camPos, CameraDir(viewer) );
 
             /* old way: draw every triangle
             for(int i=eFace::faces.Len()-1;i>=0;i--){
@@ -392,7 +446,7 @@ void eGrid::display_simple( int viewer,bool floor,
             glScalef(0.01*gs,gs,gs);
 
             se_glFloorTexture_a();
-            infinity_xy_plane(CameraPos(viewer),CameraDir(viewer));
+            infinity_xy_plane( camPos, CameraDir(viewer) );
 
             se_glFloorColor(flooralpha);
 
@@ -404,7 +458,7 @@ void eGrid::display_simple( int viewer,bool floor,
 
             glDepthFunc(GL_LEQUAL);
             glBlendFunc(GL_SRC_ALPHA,GL_ONE);
-            infinity_xy_plane(CameraPos(viewer),CameraDir(viewer));
+            infinity_xy_plane( camPos, CameraDir(viewer) );
             glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 
             break;
@@ -420,13 +474,12 @@ void eGrid::display_simple( int viewer,bool floor,
     //  glDisable(GL_TEXTURE_GEN_Q);
     //  glDisable(GL_TEXTURE_GEN_R);
 
-
-
+    glEnable(GL_DEPTH_TEST);
 
     if(eWalls){
         for(int i=se_rimWalls.Len()-1;i>=0;i--){
             su_FetchAndStoreSDLInput();
-            se_rimWalls(i)->RenderReal();
+            se_rimWalls(i)->RenderReal(cameras(viewer));
         }
 
         if (sr_lowerSky && sr_highRim){
@@ -435,7 +488,7 @@ void eGrid::display_simple( int viewer,bool floor,
             //      glEnable(GL_TEXTURE_GEN_Q);
             //      glEnable(GL_TEXTURE_GEN_R);
 
-            paint_sr_lowerSky(this, viewer,sr_upperSky);
+            paint_sr_lowerSky(this, viewer,sr_upperSky, camPos);
 
             //      glDisable(GL_TEXTURE_GEN_S);
             //      glDisable(GL_TEXTURE_GEN_T);
@@ -452,12 +505,9 @@ void eGrid::display_simple( int viewer,bool floor,
     else
         glEnable(GL_DEPTH_TEST);
 
-    if (gameObjects)
-        eGameObject::RenderAll(this, cameras(viewer));
-
     if (eWalls){
         glDisable(GL_CULL_FACE);
-        draw_eWall(this,viewer,0,zNear);
+        draw_eWall(this,viewer,0,zNear,cameras(viewer));
 
         /*
         #ifdef DEBUG
@@ -486,6 +536,9 @@ void eGrid::display_simple( int viewer,bool floor,
 
     }
 
+    if (gameObjects)
+        eGameObject::RenderAll(this, cameras(viewer));
+
     eDebugLine::Render();
 #ifdef DEBUG
 
