@@ -25,6 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
+#include "config.h"
 #include "rConsole.h"
 #include "rFont.h"
 #include "tConfiguration.h"
@@ -33,9 +34,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <fcntl.h>
 #include <sstream>
 
-#ifndef WIN32
+#if HAVE_UNISTD_H
 #include <unistd.h>
-#else
+#endif
+#ifdef WIN32
 #include <io.h>
 #include <windows.h>
 //#define fileno _fileno
@@ -44,7 +46,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
 void rConsole::DoCenterDisplay(const tString &s,REAL timeout,REAL r,REAL g,REAL b){
-    std::cerr << RemoveColors(s) << '\n';
+    std::cout << tColoredString::RemoveColors(s) << '\n';
     DisplayAtNewline();
 }
 
@@ -55,7 +57,7 @@ void sr_Unblock_stdin(){
     unblocked = true;
     stdin_descriptor=fileno(stdin);
 #ifndef WIN32
-    if (isatty(stdin_descriptor))
+    // if (isatty(stdin_descriptor))
     {
         int flag=fcntl(stdin_descriptor,F_GETFL);
         fcntl(stdin_descriptor,F_SETFL,flag | O_NONBLOCK);
@@ -69,6 +71,8 @@ static char line_in[MAXLINE+2];
 static int currentIn=0;
 
 void sr_Read_stdin(){
+    tConfItemBase::LoadPlayback( true );
+
     if ( !unblocked )
     {
         return;
