@@ -25,8 +25,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
-#include "defs.h"
+#include "rFont.h"
 
+#include "defs.h"
 
 #include <string>
 #include "rTexture.h"
@@ -35,7 +36,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "rViewport.h"
 #include "tConfiguration.h"
 #include "tSysTime.h"
-#include "rFont.h"
 
 #ifndef DEDICATED
 // #include "../network/nNetwork.h"
@@ -59,6 +59,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 //#endif
 #endif
 
+tCONFIG_ENUM( rResolution );
+tCONFIG_ENUM( rColorDepth );
+
 SDL_Surface *sr_screen=NULL; // our window
 
 #ifndef DEDICATED
@@ -66,22 +69,34 @@ static int default_texturemode = GL_LINEAR_MIPMAP_LINEAR;
 #endif
 
 bool sr_ZTrick=false;
+bool sr_useDisplayLists=false;
 
-static int width[]={320	,320,400,512,640,800,1024	,1280 ,1280	,1600	,1680 ,2048	,800};
-static int height[]={200,240,300,384,480,600,768	,854  ,1024	,1200	,1050 ,1572	,600};
-static REAL aspect[]={1	,1	,1	,1	,1	,1    ,1	,1	,1    ,1	,1		,1,1	};
+static int width[]={320	,320,400,512,640,800,1024	,1280, 	1280 ,1280	,1600	,1680 ,2048	,800,320};
+static int height[]={200,240,300,384,480,600,768	,800, 	854  ,1024	,1200	,1050 ,1572	,600,200};
+static REAL aspect[]={1	,1	,1	,1	,1	,1    ,1	,1	,1    ,1	,1		,1,1	,1};
 
 int sr_screenWidth,sr_screenHeight;
 
-static tConfItem<int>  at_ch("CUSTOM_SCREEN_HEIGHT"	, height[ArmageTron_Custom]);
-static tConfItem<int>  at_cw("CUSTOM_SCREEN_WIDTH" 	, width	[ArmageTron_Custom]);
-static tConfItem<REAL> at_ca("CUSTOM_SCREEN_ASPECT" , aspect[ArmageTron_Custom]);
-
+static tSettingItem<int>  at_ch("CUSTOM_SCREEN_HEIGHT"	, height[ArmageTron_Custom]);
+static tSettingItem<int>  at_cw("CUSTOM_SCREEN_WIDTH" 	, width	[ArmageTron_Custom]);
+static tSettingItem<REAL> at_ca("CUSTOM_SCREEN_ASPECT" , aspect[ArmageTron_Custom]);
 
 #define MAXEMERGENCY 6
 
 rScreenSettings lastSuccess(ArmageTron_640_480, false);
 
+/*
+std::ostream & operator << ( std::ostream & s, rScreenSize const & size )
+{
+    return s;
+}
+
+std::istream & operator >> ( std::istream & s, rScreenSize const & size )
+{
+    return s;
+}
+*/
+
 #ifndef DEBUG
 static rScreenSettings em6(ArmageTron_320_240, false, ArmageTron_ColorDepth_16, true, false);
 static rScreenSettings em5(ArmageTron_320_240, false, ArmageTron_ColorDepth_Desktop, true, false);
@@ -101,21 +116,33 @@ rScreenSettings currentScreensetting(ArmageTron_640_480, true);
 
 static int failed_attempts = 0;
 
-static tConfItem<int> screenres("ARMAGETRON_SCREENMODE",reinterpret_cast<int &>(currentScreensetting.res));
-static tConfItem<int> screenresLast("ARMAGETRON_LAST_SCREENMODE",
-                                    reinterpret_cast<int &>(lastSuccess.res));
+static tConfItem<rResolution> screenres("ARMAGETRON_SCREENMODE",currentScreensetting.res.res);
+static tConfItem<rResolution> screenresLast("ARMAGETRON_LAST_SCREENMODE",lastSuccess.res.res);
 
-static tConfItem<int> winsize("ARMAGETRON_WINDOWSIZE",reinterpret_cast<int &>(currentScreensetting.windowSize));
-static tConfItem<int> winsizeLast("ARMAGETRON_LAST_WINDOWSIZE",
-                                  reinterpret_cast<int &>(lastSuccess.windowSize));
+static tConfItem<rResolution> winsize("ARMAGETRON_WINDOWSIZE",currentScreensetting.windowSize.res);
+static tConfItem<rResolution> winsizeLast("ARMAGETRON_LAST_WINDOWSIZE",lastSuccess.windowSize.res);
+
+static tConfItem<int> screenres_w("ARMAGETRON_SCREENMODE_W",currentScreensetting.res.width);
+static tConfItem<int> screenresLast_w("ARMAGETRON_LAST_SCREENMODE_W", lastSuccess.res.width);
+
+static tConfItem<int> winsize_w("ARMAGETRON_WINDOWSIZE_W",currentScreensetting.windowSize.width);
+static tConfItem<int> winsizeLast_w("ARMAGETRON_LAST_WINDOWSIZE_W",lastSuccess.windowSize.width);
+
+static tConfItem<int> screenres_h("ARMAGETRON_SCREENMODE_H",currentScreensetting.res.height);
+static tConfItem<int> screenresLast_h("ARMAGETRON_LAST_SCREENMODE_H", lastSuccess.res.height);
+
+// static tConfItem<rScreenSize> winsize_wh("ARMAGETRON_WINDOWSIZE_WH",currentScreensetting.windowSize);
+
+static tConfItem<int> winsize_h("ARMAGETRON_WINDOWSIZE_H",currentScreensetting.windowSize.height);
+static tConfItem<int> winsizeLast_h("ARMAGETRON_LAST_WINDOWSIZE_H",lastSuccess.windowSize.height);
 
 static tConfItem<bool> fs_ci("FULLSCREEN",currentScreensetting.fullscreen);
 static tConfItem<bool> fs_lci("LAST_FULLSCREEN",currentScreensetting.fullscreen);
 
-static tConfItem<int> tc("COLORDEPTH",(int&)currentScreensetting.colorDepth);
-static tConfItem<int> ltc("LAST_COLORDEPTH",(int&)lastSuccess.colorDepth);
-static tConfItem<int> tzd("ZDEPTH",(int&)currentScreensetting.zDepth);
-static tConfItem<int> ltzd("LAST_ZDEPTH",(int&)lastSuccess.zDepth);
+static tConfItem<rColorDepth> tc("COLORDEPTH",currentScreensetting.colorDepth);
+static tConfItem<rColorDepth> ltc("LAST_COLORDEPTH",lastSuccess.colorDepth);
+static tConfItem<rColorDepth> tzd("ZDEPTH",currentScreensetting.zDepth);
+static tConfItem<rColorDepth> ltzd("LAST_ZDEPTH",lastSuccess.zDepth);
 
 #ifdef DIRTY
 #ifdef SDL_OPENGL
@@ -133,7 +160,7 @@ static tConfItem<int> fa("FAILED_ATTEMPTS", failed_attempts);
 
 static tCallback *rPerFrameTask_anchor;
 
-rPerFrameTask::rPerFrameTask(VOIDFUNC *f):tCallback(rPerFrameTask_anchor, f){};
+rPerFrameTask::rPerFrameTask(VOIDFUNC *f):tCallback(rPerFrameTask_anchor, f){}
 void rPerFrameTask::DoPerFrameTasks(){Exec(rPerFrameTask_anchor);}
 
 
@@ -142,11 +169,147 @@ void rPerFrameTask::DoPerFrameTasks(){Exec(rPerFrameTask_anchor);}
 static tCallbackString *RenderId_anchor;
 
 rRenderIdCallback::rRenderIdCallback(STRINGRETFUNC *f)
-        :tCallbackString(RenderId_anchor, f){};
+        :tCallbackString(RenderId_anchor, f){}
 tString rRenderIdCallback::RenderId(){return Exec(RenderId_anchor);}
 
 // *******************************************
 
+// *******************************************************************************************
+// *
+// *   rScreenSize
+// *
+// *******************************************************************************************
+//!
+//!        @param  w   screen width
+//!        @param  h  screen height
+//!
+// *******************************************************************************************
+
+rScreenSize::rScreenSize( int w, int h )
+        :res( ArmageTron_Invalid ), width(w), height(h)
+{
+}
+
+// *******************************************************************************************
+// *
+// *   rScreenSize
+// *
+// *******************************************************************************************
+//!
+//!        @param  r
+//!
+// *******************************************************************************************
+
+rScreenSize::rScreenSize( rResolution r )
+        :res( r )
+{
+    UpdateSize();
+}
+
+// *******************************************************************************************
+// *
+// *   UpdateSize
+// *
+// *******************************************************************************************
+//!
+//!
+// *******************************************************************************************
+
+void rScreenSize::UpdateSize( void )
+{
+    if ( res != ArmageTron_Invalid )
+    {
+        width = ::width[res];
+        height = ::height[res];
+        // res = ArmageTron_Invalid;
+    }
+}
+
+// *******************************************************************************************
+// *
+// *   operator ==
+// *
+// *******************************************************************************************
+//!
+//!        @param  other   size to compare with
+//!        @return true iff equal
+//!
+// *******************************************************************************************
+
+bool rScreenSize::operator ==( rScreenSize const & other ) const
+{
+    return Compare( other ) == 0;
+}
+
+// *******************************************************************************************
+// *
+// *   operator !=
+// *
+// *******************************************************************************************
+//!
+//!        @param  other   size to compare with
+//!        @return  true iff not equal
+//!
+// *******************************************************************************************
+
+bool rScreenSize::operator !=( rScreenSize const & other ) const
+{
+    return Compare( other ) != 0;
+}
+
+// *******************************************************************************************
+// *
+// *   Compare
+// *
+// *******************************************************************************************
+//!
+//!        @param  other   size to compare with
+//!        @return         0 if eqal, -1 if this is smaller, +1 if other is smaller
+//!
+// *******************************************************************************************
+
+int rScreenSize::Compare( rScreenSize const & other ) const
+{
+    if ( width < other.width )
+        return -1;
+    else if ( width > other.width )
+        return 1;
+
+    if ( height < other.height )
+        return -1;
+    else if ( height > other.height )
+        return 1;
+
+    /* res is not really a criterion, ignore it
+    if ( res < other.res )
+        return -1;
+    else if ( res > other.res )
+        return 1;
+    */
+
+    return 0;
+}
+
+
+// *******************************************************************************************
+// *
+// *   rScreenSettings
+// *
+// *******************************************************************************************
+//!
+//!        @param  r   the resolution
+//!        @param  fs  fullscreen flag
+//!        @param  cd  color depth
+//!        @param  sdl use clean sdl initialization
+//!        @param  ce  check for errors
+//!
+// *******************************************************************************************
+
+rScreenSettings::rScreenSettings( rResolution r, bool fs, rColorDepth cd, bool sdl, bool ce )
+        :res(r), windowSize(r), fullscreen(fs), colorDepth(cd), zDepth( ArmageTron_ColorDepth_Desktop ), useSDL(sdl), checkErrors(true), aspect (1)
+{
+}
+
 void sr_ReinitDisplay(){
     sr_ExitDisplay();
     if (!sr_InitDisplay()){
@@ -174,7 +337,7 @@ bool last_software_renderer=false;
 
 static tConfItem<bool> lsr("SOFTWARE_RENDERER",last_software_renderer);
 
-tString lastError = "Unknown";
+tString lastError("Unknown");
 
 #ifndef DEDICATED
 static int countBits(unsigned int count)
@@ -192,17 +355,18 @@ static int countBits(unsigned int count)
 
 static bool lowlevel_sr_InitDisplay(){
 #ifndef DEDICATED
-    rResolution res = currentScreensetting.fullscreen ? currentScreensetting.res : currentScreensetting.windowSize;
+    rScreenSize & res = currentScreensetting.fullscreen ? currentScreensetting.res : currentScreensetting.windowSize;
 
     // update pixel aspect ratio
-    currentScreensetting.aspect = aspect[res];
+    if ( res.res != ArmageTron_Invalid )
+        currentScreensetting.aspect = aspect[res.res];
 
 #ifndef DIRTY
     currentScreensetting.useSDL = true;
 #endif
-
-    sr_screenWidth =width[res];
-    sr_screenHeight=height[res];
+    res.UpdateSize();
+    sr_screenWidth = res.width;
+    sr_screenHeight= res.height;
 
     if (!sr_screen)
     {
@@ -399,7 +563,7 @@ static bool lowlevel_sr_InitDisplay(){
 
 #ifdef DIRTY
     if (!currentScreensetting.useSDL)
-        if(!sr_InitGL()) return false;
+        if(!rSysDep::InitGL()) return false;
 #endif
 
 #ifndef DEDICATED
@@ -470,9 +634,13 @@ static bool lowlevel_sr_InitDisplay(){
     // disable storage of non-alpha textures on Savage MX
     if ( strstr( gl_renderer, "SavageMX" ) )
     {
-        sr_storageHack = true;
+        rISurfaceTexture::storageHack_ = true;
     }
 
+    // fonts look best in bilinear filtering, no mipmaps
+    if ( rTextureGroups::TextureMode[rTextureGroups::TEX_FONT] > GL_LINEAR )
+        rTextureGroups::TextureMode[rTextureGroups::TEX_FONT] = GL_LINEAR;
+
     // disable trilinear filtering for ATI cards
     if ( strstr( gl_vendor, "ATI" ) )
     {
@@ -500,7 +668,10 @@ static bool lowlevel_sr_InitDisplay(){
     return true;
 }
 
+extern bool cycleprograminited;
+
 bool sr_InitDisplay(){
+    cycleprograminited = false;
     while (failed_attempts <= MAXEMERGENCY+1)
     {
 #ifndef DEBUG
@@ -574,7 +745,7 @@ void sr_ExitDisplay(){
     rCallbackBeforeScreenModeChange::Exec();
 
 #ifdef DIRTY
-    sr_ExitGL();
+    rSysDep::ExitGL();
 #endif
 
     if (sr_screen){
@@ -616,6 +787,8 @@ bool sr_texturesTruecolor=false;
 bool sr_textOut=false;
 bool sr_FPSOut=true;
 
+bool sr_keepWindowActive=false;
+
 tString renderer_identification;
 
 void sr_LoadDefaultConfig(){
@@ -626,28 +799,34 @@ void sr_LoadDefaultConfig(){
     sr_perspectiveCorrection=rFEAT_DEFAULT;
     sr_alphaBlend=true;
     sr_ZTrick=false;
+    sr_useDisplayLists=false;
     sr_textOut=true;
     sr_dither=true;
     sr_smoothShading=true;
     int i;
 #ifndef DEDICATED
-    for (i=rTEX_GROUPS-1;i>=0;i--)
-        TextureMode[i]=default_texturemode;
+    for (i=rTextureGroups::TEX_GROUPS-1;i>=0;i--)
+        rTextureGroups::TextureMode[i]=default_texturemode;
+
+    // fonts look best in bilinear filtering, no mipmaps
+    rTextureGroups::TextureMode[rTextureGroups::TEX_FONT]=GL_LINEAR;
 #endif
     sr_floorDetail=rFLOOR_TWOTEXTURE;
     sr_floorMirror=rMIRROR_OFF;
     sr_infinityPlane=false;
     sr_lowerSky=false;
     sr_upperSky=false;
+    sr_keepWindowActive=false;
+    rSysDep::swapMode_=rSysDep::rSwap_glFinish;
 
     if (software_renderer){
         // A software renderer! Poor soul. Set low details:
-        for (i=rTEX_GROUPS-1;i>=0;i--)
-            TextureMode[i]=-1;
+        for (i=rTextureGroups::TEX_GROUPS-1;i>=0;i--)
+            rTextureGroups::TextureMode[i]=-1;
 
 #ifndef DEDICATED
-        TextureMode[rTEX_OBJ]=GL_NEAREST_MIPMAP_NEAREST;
-        TextureMode[rTEX_FONT]=GL_NEAREST_MIPMAP_NEAREST;
+        rTextureGroups::TextureMode[rTextureGroups::TEX_OBJ]=GL_NEAREST_MIPMAP_NEAREST;
+        rTextureGroups::TextureMode[rTextureGroups::TEX_FONT]=GL_NEAREST_MIPMAP_NEAREST;
 #endif
 
         sr_lineAntialias=rFEAT_OFF;
@@ -668,9 +847,17 @@ void sr_LoadDefaultConfig(){
         sr_perspectiveCorrection=rFEAT_ON;
     }
     else if(strstr(gl_vendor,"NVIDIA")){
-        // infinity works for NVIDIA
+        // infinity , display lists and glFlush swapping work for NVIDIA
         sr_infinityPlane=true;
+        sr_useDisplayLists=true;
+        rSysDep::swapMode_=rSysDep::rSwap_glFlush;
     }
+#ifdef MACOSX
+    else if(strstr(gl_vendor,"ATI")){
+        // glFlush swapping work for ATI on the mac
+        rSysDep::swapMode_=rSysDep::rSwap_glFlush;
+    }
+#endif
     else if(strstr(gl_vendor,"Matrox")){
         sr_floorDetail = rFLOOR_TEXTURE;  // double textured floor does not work
     }
@@ -818,13 +1005,27 @@ void sr_DepthOffset(bool offset){
 void sr_Activate(bool active)
 {
 #ifndef DEDICATED
-    sr_glOut=active;
+    if ( !currentScreensetting.fullscreen && !active && sr_keepWindowActive )
+    {
+        sr_glOut=!active;
+    }
+    else
+    {
+        sr_glOut=active;
+    }
 
+    // unload textures and stuff if rendering gets disabled
+    if (!sr_glOut)
+        rCallbackBeforeScreenModeChange::Exec();
+
+    // Jonathans fullscreen bugfix.
+#ifndef MACOSX
     if ( currentScreensetting.fullscreen && !active )
     {
         SDL_WM_IconifyWindow();
     }
 #endif
+#endif
 }
 
 //**************************************
